@@ -37,7 +37,6 @@ import org.xnio.conduits.ConduitStreamSinkChannel;
 
 import java.io.IOException;
 import java.util.Date;
-import java.util.Map;
 import java.util.concurrent.Executor;
 import java.util.concurrent.RejectedExecutionException;
 
@@ -48,10 +47,12 @@ import java.util.concurrent.RejectedExecutionException;
  * by connector implementations.
  *
  * @author Stuart Douglas
+ * @author <a href="mailto:ropalka@redhat.com">Richard Opalka</a>
  */
 public class Connectors {
 
     private static final boolean[] ALLOWED_TOKEN_CHARACTERS = new boolean[256];
+    private static final boolean[] ALLOWED_SCHEME_CHARACTERS = new boolean[256];
 
     static {
         for(int i = 0; i < ALLOWED_TOKEN_CHARACTERS.length; ++i) {
@@ -84,6 +85,25 @@ public class Connectors {
                 }
             }
         }
+
+        for(int i = 0; i < ALLOWED_SCHEME_CHARACTERS.length; ++i) {
+            if((i >='0' && i <= '9') ||
+                    (i >='a' && i <= 'z') ||
+                    (i >='A' && i <= 'Z')) {
+                ALLOWED_SCHEME_CHARACTERS[i] = true;
+            } else {
+                switch (i) {
+                    case '+':
+                    case '-':
+                    case '.': {
+                        ALLOWED_SCHEME_CHARACTERS[i] = true;
+                        break;
+                    }
+                    default:
+                        ALLOWED_SCHEME_CHARACTERS[i] = false;
+                }
+            }
+        }
     }
     /**
      * Flattens the exchange cookie map into the response header map. This should be called by a
@@ -92,12 +112,9 @@ public class Connectors {
      * @param exchange The server exchange
      */
     public static void flattenCookies(final HttpServerExchange exchange) {
-        Map<String, Cookie> cookies = exchange.getResponseCookiesInternal();
         boolean enableRfc6265Validation = exchange.getConnection().getUndertowOptions().get(UndertowOptions.ENABLE_RFC6265_COOKIE_VALIDATION, UndertowOptions.DEFAULT_ENABLE_RFC6265_COOKIE_VALIDATION);
-        if (cookies != null) {
-            for (Map.Entry<String, Cookie> entry : cookies.entrySet()) {
-                exchange.getResponseHeaders().add(Headers.SET_COOKIE, getCookieString(entry.getValue(), enableRfc6265Validation));
-            }
+        for (Cookie cookie : exchange.responseCookies()) {
+            exchange.getResponseHeaders().add(Headers.SET_COOKIE, getCookieString(cookie, enableRfc6265Validation));
         }
     }
 
@@ -243,8 +260,6 @@ public class Connectors {
             if (cookie.getSameSiteMode() != null && !cookie.getSameSiteMode().isEmpty()) {
                 header.append("; SameSite=");
                 header.append(cookie.getSameSiteMode());
-            } else {
-                header.append("; SameSite");
             }
         }
         return header.toString();
@@ -295,8 +310,6 @@ public class Connectors {
             if (cookie.getSameSiteMode() != null && !cookie.getSameSiteMode().isEmpty()) {
                 header.append("; SameSite=");
                 header.append(cookie.getSameSiteMode());
-            } else {
-                header.append("; SameSite");
             }
         }
         return header.toString();
@@ -363,8 +376,6 @@ public class Connectors {
             if (cookie.getSameSiteMode() != null && !cookie.getSameSiteMode().isEmpty()) {
                 header.append("; SameSite=");
                 header.append(cookie.getSameSiteMode());
-            } else {
-                header.append("; SameSite");
             }
         }
         return header.toString();
@@ -441,58 +452,53 @@ public class Connectors {
          */
     public static void setExchangeRequestPath(final HttpServerExchange exchange, final String encodedPath, final String charset, boolean decode, final boolean allowEncodedSlash, StringBuilder decodeBuffer, int maxParameters) throws ParameterLimitException {
         boolean requiresDecode = false;
+        final StringBuilder pathBuilder = new StringBuilder();
+        int currentPathPartIndex = 0;
         for (int i = 0; i < encodedPath.length(); ++i) {
             char c = encodedPath.charAt(i);
             if (c == '?') {
                 String part;
-                String encodedPart = encodedPath.substring(0, i);
+                String encodedPart = encodedPath.substring(currentPathPartIndex, i);
                 if (requiresDecode) {
                     part = URLUtils.decode(encodedPart, charset, allowEncodedSlash,false, decodeBuffer);
                 } else {
                     part = encodedPart;
                 }
+                pathBuilder.append(part);
+                part = pathBuilder.toString();
                 exchange.setRequestPath(part);
                 exchange.setRelativePath(part);
-                exchange.setRequestURI(encodedPart);
+                exchange.setRequestURI(encodedPath.substring(0, i));
                 final String qs = encodedPath.substring(i + 1);
                 exchange.setQueryString(qs);
                 URLUtils.parseQueryString(qs, exchange, charset, decode, maxParameters);
                 return;
             } else if(c == ';') {
                 String part;
-                String encodedPart = encodedPath.substring(0, i);
+                String encodedPart = encodedPath.substring(currentPathPartIndex, i);
                 if (requiresDecode) {
                     part = URLUtils.decode(encodedPart, charset, allowEncodedSlash, false, decodeBuffer);
                 } else {
                     part = encodedPart;
                 }
-                exchange.setRequestPath(part);
-                exchange.setRelativePath(part);
-                for(int j = i; j < encodedPath.length(); ++j) {
-                    if (encodedPath.charAt(j) == '?') {
-                        exchange.setRequestURI(encodedPath.substring(0, j));
-                        String pathParams = encodedPath.substring(i + 1, j);
-                        URLUtils.parsePathParams(pathParams, exchange, charset, decode, maxParameters);
-                        String qs = encodedPath.substring(j + 1);
-                        exchange.setQueryString(qs);
-                        URLUtils.parseQueryString(qs, exchange, charset, decode, maxParameters);
-                        return;
-                    }
-                }
+                pathBuilder.append(part);
                 exchange.setRequestURI(encodedPath);
-                URLUtils.parsePathParams(encodedPath.substring(i + 1), exchange, charset, decode, maxParameters);
-                return;
+                currentPathPartIndex = i + 1 + URLUtils.parsePathParams(encodedPath.substring(i + 1), exchange, charset, decode, maxParameters);
+                i = currentPathPartIndex -1 ;
             } else if(c == '%' || c == '+') {
                 requiresDecode = decode;
             }
         }
 
         String part;
+        String encodedPart = encodedPath.substring(currentPathPartIndex);
         if (requiresDecode) {
-            part = URLUtils.decode(encodedPath, charset, allowEncodedSlash, false, decodeBuffer);
+            part = URLUtils.decode(encodedPart, charset, allowEncodedSlash, false, decodeBuffer);
         } else {
-            part = encodedPath;
+            part = encodedPart;
         }
+        pathBuilder.append(part);
+        part = pathBuilder.toString();
         exchange.setRequestPath(part);
         exchange.setRelativePath(part);
         exchange.setRequestURI(encodedPath);
@@ -552,6 +558,10 @@ public class Connectors {
         return ALLOWED_TOKEN_CHARACTERS[c];
     }
 
+    public static boolean isValidSchemeCharacter(byte c) {
+        return ALLOWED_SCHEME_CHARACTERS[c];
+    }
+
 
     /**
      * Verifies that the provided request headers are valid according to rfc7230. In particular:
