@@ -228,12 +228,15 @@ public final class HttpServerExchange extends AbstractAttachable {
      * The default value for this is determined by the {@link io.undertow.UndertowOptions#MAX_ENTITY_SIZE} option. A value
      * of 0 indicates that this is unbounded.
      * <p>
+     * In case of multipart handling, this will default to {@link io.undertow.UndertowOptions#MULTIPART_MAX_ENTITY_SIZE}
+     * <p>
      * If this entity size is exceeded the request channel will be forcibly closed.
      * <p>
      * TODO: integrate this with HTTP 100-continue responses, to make it possible to send a 417 rather than just forcibly
      * closing the channel.
      *
      * @see io.undertow.UndertowOptions#MAX_ENTITY_SIZE
+     * @see io.undertow.UndertowOptions#MULTIPART_MAX_ENTITY_SIZE
      */
     private long maxEntitySize;
 
@@ -570,8 +573,16 @@ public final class HttpServerExchange extends AbstractAttachable {
         return queryString;
     }
 
+    /**
+     * Set query string.  Leading ? char will be removed automatically.
+     */
     public HttpServerExchange setQueryString(final String queryString) {
-        this.queryString = queryString;
+        // Clean leading ?
+        if( queryString.length() > 0 && queryString.charAt(0) == '?' ) {
+                this.queryString = queryString.substring(1);
+            } else {
+                this.queryString = queryString;
+            }
         return this;
     }
 
