@@ -48,7 +48,7 @@ import java.util.ServiceLoader;
 
 /**
  * Parser for the undertow-handlers.conf file.
- * <p/>
+ * <p>
  * This file has a line by line syntax, specifying predicate -&gt; handler. If no predicate is specified then
  * the line is assumed to just contain a handler.
  *
@@ -136,6 +136,9 @@ public class PredicatedHandlersParser {
         } else if(node instanceof BlockNode) {
             List<PredicatedHandler> handlers = handleBlockNode(contents, (BlockNode) node, predicateBuilders, handlerBuilders, parser);
             return  new PredicatesHandler.Wrapper(handlers, false);
+        } else if(node instanceof PredicateOperatorNode) {
+            List<PredicatedHandler> handlers = Collections.singletonList(handlePredicateOperatorNode(contents, (PredicateOperatorNode)node, predicateBuilders, handlerBuilders, parser));
+            return  new PredicatesHandler.Wrapper(handlers, false);
         } else {
             throw error(contents, node.getToken().getPosition(), "unexpected token " + node.getToken());
         }
@@ -343,6 +346,10 @@ public class PredicatedHandlersParser {
                 }
                 break;
             } else if(token.getToken().equals("\n") || token.getToken().equals(";")) {
+                if(token.getToken().equals(";") && tokens.peek()!=null && tokens.peek().getToken().equals(ELSE)) {
+                    // something() -> predicate; ELSE predicate; - dont end processing since its followed by ELSE and its singular block
+                    continue;
+                }
                 handleLineEnd(string, operatorStack, output, blocks);
             } else if (isSpecialChar(token.getToken())) {
                 if (token.getToken().equals("(")) {
