@@ -99,12 +99,12 @@ public class ResourceHandler implements HttpHandler {
     private volatile ResourceManager resourceManager;
     /**
      * If this is set this will be the maximum time (in seconds) the client will cache the resource.
-     * <p/>
+     * <p>
      * Note: Do not set this for private resources, as it will cause a Cache-Control: public
      * to be sent.
-     * <p/>
+     * <p>
      * TODO: make this more flexible
-     * <p/>
+     * <p>
      * This will only be used if the {@link #cachable} predicate returns true
      */
     private volatile Integer cacheTime;
@@ -155,7 +155,7 @@ public class ResourceHandler implements HttpHandler {
             if (KNOWN_METHODS.contains(exchange.getRequestMethod())) {
                 exchange.setStatusCode(StatusCodes.METHOD_NOT_ALLOWED);
                 exchange.getResponseHeaders().add(Headers.ALLOW,
-                        join(", ", Methods.GET_STRING, Methods.HEAD_STRING, Methods.POST_STRING));
+                        String.join(", ", Methods.GET_STRING, Methods.HEAD_STRING, Methods.POST_STRING));
             } else {
                 exchange.setStatusCode(StatusCodes.NOT_IMPLEMENTED);
             }
@@ -373,20 +373,6 @@ public class ResourceHandler implements HttpHandler {
         return s;
     }
 
-    private static String join(String delimiter, String... items) {
-        StringBuilder sb = new StringBuilder();
-        boolean first = true;
-        for (String item: items) {
-            if (first) {
-                first = false;
-            } else {
-                sb.append(delimiter);
-            }
-            sb.append(item);
-        }
-        return sb.toString();
-    }
-
     public boolean isDirectoryListingEnabled() {
         return directoryListingEnabled;
     }
