@@ -83,6 +83,7 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
     private long frameDataRemaining;
 
     private final Object lock = new Object();
+    // Guarded by lock
     private int waiters;
     private volatile boolean waitingForFrame;
     private int readFrameCount = 0;
@@ -329,7 +330,7 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
         waitingForFrame = false;
         if(data == null && pendingFrameData.isEmpty() && frameDataRemaining == 0) {
             synchronized (lock) {
-                state |= STATE_DONE | STATE_CLOSED;
+                state |= STATE_DONE;
             }
             getFramedChannel().notifyFrameReadComplete(this);
             IoUtils.safeClose(this);
@@ -345,9 +346,9 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
         if(Thread.currentThread() == getIoThread()) {
             throw UndertowMessages.MESSAGES.awaitCalledFromIoThread();
         }
-        if (data == null && pendingFrameData.isEmpty()) {
+        if (data == null && pendingFrameData.isEmpty() && !anyAreSet(state, STATE_STREAM_BROKEN | STATE_CLOSED)) {
             synchronized (lock) {
-                if (data == null && pendingFrameData.isEmpty()) {
+                if (data == null && pendingFrameData.isEmpty() && !anyAreSet(state, STATE_STREAM_BROKEN | STATE_CLOSED)) {
                     try {
                         waiters++;
                         lock.wait();
@@ -367,9 +368,9 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
         if(Thread.currentThread() == getIoThread()) {
             throw UndertowMessages.MESSAGES.awaitCalledFromIoThread();
         }
-        if (data == null) {
+        if (data == null && pendingFrameData.isEmpty() && !anyAreSet(state, STATE_STREAM_BROKEN | STATE_CLOSED)) {
             synchronized (lock) {
-                if (data == null) {
+                if (data == null && pendingFrameData.isEmpty() && !anyAreSet(state, STATE_STREAM_BROKEN | STATE_CLOSED)) {
                     try {
                         waiters++;
                         lock.wait(timeUnit.toMillis(l));
@@ -622,11 +623,15 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
     }
 
     @Override
-    public synchronized void close() {
+    public void close() {
         if(anyAreSet(state, STATE_CLOSED)) {
             return;
         }
         synchronized (lock) {
+            // Double check to avoid executing the the rest of this method multiple times
+            if(anyAreSet(state, STATE_CLOSED)) {
+                return;
+            }
             state |= STATE_CLOSED;
             if (allAreClear(state, STATE_DONE | STATE_LAST_FRAME)) {
                 state |= STATE_STREAM_BROKEN;
@@ -646,6 +651,11 @@ public abstract class AbstractFramedStreamSourceChannel<C extends AbstractFramed
                     closeListeners[i].handleEvent(this);
                 }
             }
+            // UNDERTOW-1639: Close may be called from an I/O thread while a worker is blocked on awaitReadable.
+            // Once the channel is closed, callers must be awoken.
+            if (waiters > 0) {
+                lock.notifyAll();
+            }
         }
     }
 
