@@ -81,6 +81,11 @@ class ParseState {
      */
     final StringBuilder stringBuilder = new StringBuilder();
 
+    /**
+     * We need to keep track of the canonical path
+     */
+    final StringBuilder canonicalPath = new StringBuilder();
+
     /**
      * This has different meanings depending on the current state.
      *
