@@ -132,7 +132,14 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
     @Override
     public void stop() {
         for (Map.Entry<String, SessionImpl> session : sessions.entrySet()) {
-            session.getValue().destroy();
+            final SessionImpl sessionValue = session.getValue();
+            sessionValue.destroy();
+            if (sessionValue.getId() == null) {
+                // this means we are creating the session right now in a different thread,
+                // do not send the session to listener with a null id,
+                // just set it again, setting the same session id twice is harmless
+                sessionValue.setId(session.getKey());
+            }
             sessionListeners.sessionDestroyed(session.getValue(), null, SessionListener.SessionDestroyedReason.UNDEPLOY);
         }
         sessions.clear();
@@ -145,13 +152,16 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                 while (sessions.size() >= maxSize && !evictionQueue.isEmpty()) {
 
                     String key = evictionQueue.poll();
+                    if(key == null) {
+                        break;
+                    }
                     UndertowLogger.REQUEST_LOGGER.debugf("Removing session %s as max size has been hit", key);
                     SessionImpl toRemove = sessions.get(key);
                     if (toRemove != null) {
                         toRemove.invalidate(null, SessionListener.SessionDestroyedReason.TIMEOUT); //todo: better reason
                     }
                 }
-            } else if(sessions.size() >= maxSize) {
+            } else if (sessions.size() >= maxSize) {
                 if(statisticsEnabled) {
                     rejectedSessionCount.incrementAndGet();
                 }
@@ -161,29 +171,21 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         if (config == null) {
             throw UndertowMessages.MESSAGES.couldNotFindSessionCookieConfig();
         }
-        String sessionID = config.findSessionId(serverExchange);
-        int count = 0;
-        while (sessionID == null) {
-            sessionID = sessionIdGenerator.createSessionId();
-            if(sessions.containsKey(sessionID)) {
-                sessionID = null;
-            }
-            if(count++ == 100) {
-                //this should never happen
-                //but we guard against pathalogical session id generators to prevent an infinite loop
-                throw UndertowMessages.MESSAGES.couldNotGenerateUniqueSessionId();
-            }
-        }
-        Object evictionToken;
-        if (evictionQueue != null) {
-            evictionToken = evictionQueue.offerLastAndReturnToken(sessionID);
-        } else {
-            evictionToken = null;
-        }
-        final SessionImpl session = new SessionImpl(this, sessionID, config, serverExchange.getIoThread(), serverExchange.getConnection().getWorker(), evictionToken, defaultSessionTimeout);
 
+        String sessionID = config.findSessionId(serverExchange);
+        final SessionImpl session = new SessionImpl(this, config, serverExchange.getIoThread(), serverExchange.getConnection().getWorker(), defaultSessionTimeout);
+        if (sessionID != null) {
+            if (!saveSessionID(sessionID, session))
+                throw UndertowMessages.MESSAGES.sessionWithIdAlreadyExists(sessionID);
+            // else: succeeded to use requested session id
+        } else {
+            sessionID = createAndSaveNewID(session);
+        }
+        session.setId(sessionID);
+        if (evictionQueue != null) {
+            session.setEvictionToken(evictionQueue.offerLastAndReturnToken(sessionID));
+        }
         UndertowLogger.SESSION_LOGGER.debugf("Created session with id %s for exchange %s", sessionID, serverExchange);
-        sessions.put(sessionID, session);
         config.setSessionId(serverExchange, session.getId());
         session.bumpTimeout();
         sessionListeners.sessionCreated(session, serverExchange);
@@ -201,9 +203,25 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                 }
             } while (!highestSessionCount.compareAndSet(highest, sessionSize));
         }
+
         return session;
     }
 
+    private boolean saveSessionID(String sessionID, SessionImpl session) {
+        return this.sessions.putIfAbsent(sessionID, session) == null;
+    }
+
+    private String createAndSaveNewID(SessionImpl session) {
+        for (int i = 0; i < 100; i++) {
+            final String sessionID = sessionIdGenerator.createSessionId();
+            if (saveSessionID(sessionID, session))
+                return sessionID;
+        }
+        //this should 'never' happen
+        //but we guard against pathological session id generators to prevent an infinite loop
+        throw UndertowMessages.MESSAGES.couldNotGenerateUniqueSessionId();
+    }
+
     @Override
     public Session getSession(final HttpServerExchange serverExchange, final SessionConfig config) {
         if (serverExchange != null) {
@@ -228,12 +246,16 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         final SessionImpl sess = sessions.get(sessionId);
         if (sess == null) {
             return null;
-        } else {
-            return sess;
         }
+        if (sess.getId() == null) {
+            // this means we are creating the session right now in a different thread,
+            // do not return the session with a null id to the outer world,
+            // just set it again, setting the same session id twice is harmless
+            sess.setId(sessionId);
+        }
+        return sess;
     }
 
-
     @Override
     public synchronized void registerSessionListener(final SessionListener listener) {
         UndertowLogger.SESSION_LOGGER.debugf("Registered session listener %s", listener);
@@ -370,7 +392,7 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         }
 
 
-        private String sessionId;
+        private volatile String sessionId;
         private volatile Object evictionToken;
         private final SessionConfig sessionCookieConfig;
         private volatile long expireTime = -1;
@@ -399,15 +421,13 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             }
         };
 
-        private SessionImpl(final InMemorySessionManager sessionManager, final String sessionId, final SessionConfig sessionCookieConfig, final XnioIoThread executor, final XnioWorker worker, final Object evictionToken, final int maxInactiveInterval) {
+        private SessionImpl(final InMemorySessionManager sessionManager, final SessionConfig sessionCookieConfig, final XnioIoThread executor, final XnioWorker worker, final int maxInactiveInterval) {
             this.sessionManager = sessionManager;
-            this.sessionId = sessionId;
             this.sessionCookieConfig = sessionCookieConfig;
             this.executor = executor;
             this.worker = worker;
-            this.evictionToken = evictionToken;
             creationTime = lastAccessed = System.currentTimeMillis();
-            this.maxInactiveInterval = maxInactiveInterval;
+            this.setMaxInactiveInterval(maxInactiveInterval);
         }
 
         synchronized void bumpTimeout() {
@@ -415,9 +435,9 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                 return;
             }
 
-            final int maxInactiveInterval = getMaxInactiveInterval();
+            final long maxInactiveInterval = getMaxInactiveIntervalMilis();
             if (maxInactiveInterval > 0) {
-                long newExpireTime = System.currentTimeMillis() + (maxInactiveInterval * 1000L);
+                long newExpireTime = System.currentTimeMillis() + maxInactiveInterval;
                 if(timerCancelKey != null && (newExpireTime < expireTime)) {
                     // We have to re-schedule as the new maxInactiveInterval is lower than the old one
                     if (!timerCancelKey.remove()) {
@@ -431,7 +451,7 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                     //+1, to make sure that the time has actually expired
                     //we don't re-schedule every time, as it is expensive
                     //instead when it expires we check if the timeout has been bumped, and if so we re-schedule
-                    timerCancelKey = executor.executeAfter(cancelTask, (maxInactiveInterval * 1000L) + 1L, TimeUnit.MILLISECONDS);
+                    timerCancelKey = executor.executeAfter(cancelTask, maxInactiveInterval + 1L, TimeUnit.MILLISECONDS);
                 }
             } else {
                 expireTime = -1;
@@ -440,6 +460,10 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                     timerCancelKey = null;
                 }
             }
+        }
+
+        private void setEvictionToken(Object evictionToken) {
+            this.evictionToken = evictionToken;
             if (evictionToken != null) {
                 Object token = evictionToken;
                 if (evictionTokenUpdater.compareAndSet(this, token, null)) {
@@ -449,6 +473,9 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             }
         }
 
+        private void setId(final String sessionId) {
+            this.sessionId = sessionId;
+        }
 
         @Override
         public String getId() {
@@ -462,6 +489,7 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                     serverExchange.putAttachment(FIRST_REQUEST_ACCESS, System.currentTimeMillis());
                 }
             }
+            bumpTimeout();
         }
 
         @Override
@@ -470,6 +498,7 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             if(existing != null) {
                 lastAccessed = existing;
             }
+            bumpTimeout();
         }
 
         @Override
@@ -494,8 +523,8 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
                 throw UndertowMessages.MESSAGES.sessionIsInvalid(sessionId);
             }
             UndertowLogger.SESSION_LOGGER.debugf("Setting max inactive interval for %s to %s", sessionId, interval);
-            maxInactiveInterval = interval;
-            bumpTimeout();
+            this.maxInactiveInterval = interval;
+            this.bumpTimeout();
         }
 
         @Override
@@ -506,12 +535,18 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             return maxInactiveInterval;
         }
 
+        private long getMaxInactiveIntervalMilis() {
+            if (invalid) {
+                throw UndertowMessages.MESSAGES.sessionIsInvalid(sessionId);
+            }
+            return this.maxInactiveInterval*1000L;
+        }
+
         @Override
         public Object getAttribute(final String name) {
             if (invalid) {
                 throw UndertowMessages.MESSAGES.sessionIsInvalid(sessionId);
             }
-            bumpTimeout();
             return attributes.get(name);
         }
 
@@ -520,7 +555,6 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             if (invalid) {
                 throw UndertowMessages.MESSAGES.sessionIsInvalid(sessionId);
             }
-            bumpTimeout();
             return attributes.keySet();
         }
 
@@ -538,7 +572,6 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             } else {
                sessionManager.sessionListeners.attributeUpdated(this, name, value, existing);
             }
-            bumpTimeout();
             UndertowLogger.SESSION_LOGGER.tracef("Setting session attribute %s to %s for session %s", name, value, sessionId);
             return existing;
         }
@@ -550,7 +583,6 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
             }
             final Object existing = attributes.remove(name);
             sessionManager.sessionListeners.attributeRemoved(this, name, existing);
-            bumpTimeout();
             UndertowLogger.SESSION_LOGGER.tracef("Removing session attribute %s for session %s", name, sessionId);
             return existing;
         }
@@ -558,7 +590,7 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         @Override
         public void invalidate(final HttpServerExchange exchange) {
             invalidate(exchange, SessionListener.SessionDestroyedReason.INVALIDATED);
-            if(exchange != null) {
+            if (exchange != null) {
                 exchange.removeAttachment(sessionManager.NEW_SESSION);
             }
             Object evictionToken = this.evictionToken;
@@ -609,10 +641,9 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         @Override
         public String changeSessionId(final HttpServerExchange exchange, final SessionConfig config) {
             final String oldId = sessionId;
-            String newId = sessionManager.sessionIdGenerator.createSessionId();
+            String newId = sessionManager.createAndSaveNewID(this);
             this.sessionId = newId;
             if(!invalid) {
-                sessionManager.sessions.put(newId, this);
                 config.setSessionId(exchange, this.getId());
             }
             sessionManager.sessions.remove(oldId);
