@@ -105,6 +105,13 @@ public final class HttpString implements Comparable<HttpString>, Serializable {
 
     HttpString(final String string, int orderInt) {
         this.orderInt = orderInt;
+        this.bytes = toByteArray(string);
+        this.hashCode = calcHashCode(bytes);
+        this.string = string;
+        checkForNewlines();
+    }
+
+    private static byte[] toByteArray(final String string) {
         final int len = string.length();
         final byte[] bytes = new byte[len];
         for (int i = 0; i < len; i++) {
@@ -114,10 +121,7 @@ public final class HttpString implements Comparable<HttpString>, Serializable {
             }
             bytes[i] = (byte) c;
         }
-        this.bytes = bytes;
-        this.hashCode = calcHashCode(bytes);
-        this.string = string;
-        checkForNewlines();
+        return bytes;
     }
 
     private void checkForNewlines() {
@@ -343,7 +347,7 @@ public final class HttpString implements Comparable<HttpString>, Serializable {
     @SuppressWarnings("deprecation")
     public String toString() {
         if (string == null) {
-            string = new String(bytes, 0);
+            string = new String(bytes, java.nio.charset.StandardCharsets.US_ASCII);
         }
         return string;
     }
@@ -357,13 +361,9 @@ public final class HttpString implements Comparable<HttpString>, Serializable {
         }
     }
 
-    static int hashCodeOf(String headerName) {
-        int hc = 17;
-
-        for (int i = 0; i < headerName.length(); ++i) {
-            hc = (hc << 4) + hc + higher((byte) headerName.charAt(i));
-        }
-        return hc;
+    static int hashCodeOf(final String headerName) {
+        final byte[] bytes = toByteArray(headerName);
+        return calcHashCode(bytes);
     }
 
     public boolean equalToString(String headerName) {
