@@ -398,11 +398,6 @@ public class PortableConcurrentDirectDeque<E>
      * Unlinks non-null node x.
      */
     void unlink(Node<E> x) {
-        // assert x != null;
-        // assert x.item == null;
-        // assert x != PREV_TERMINATOR;
-        // assert x != NEXT_TERMINATOR;
-
         final Node<E> prev = x.prev;
         final Node<E> next = x.next;
         if (prev == null) {
@@ -478,7 +473,7 @@ public class PortableConcurrentDirectDeque<E>
             // TODO: better HOP heuristics
             if (hops < HOPS
                 // always squeeze out interior deleted nodes
-                && (isFirst | isLast))
+                && (isFirst || isLast))
                 return;
 
             // Squeeze out deleted nodes between activePred and
@@ -487,7 +482,7 @@ public class PortableConcurrentDirectDeque<E>
             skipDeletedPredecessors(activeSucc);
 
             // Try to gc-unlink, if possible
-            if ((isFirst | isLast) &&
+            if ((isFirst || isLast) &&
 
                 // Recheck expected state of predecessor and successor
                 (activePred.next == activeSucc) &&
@@ -509,9 +504,6 @@ public class PortableConcurrentDirectDeque<E>
      * Unlinks non-null first node.
      */
     private void unlinkFirst(Node<E> first, Node<E> next) {
-        // assert first != null;
-        // assert next != null;
-        // assert first.item == null;
         for (Node<E> o = null, p = next, q;;) {
             if (p.item != null || (q = p.next) == null) {
                 if (o != null && p.prev != p && first.casNext(next, p)) {
@@ -543,9 +535,6 @@ public class PortableConcurrentDirectDeque<E>
      * Unlinks non-null last node.
      */
     private void unlinkLast(Node<E> last, Node<E> prev) {
-        // assert last != null;
-        // assert prev != null;
-        // assert last.item == null;
         for (Node<E> o = null, p = prev, q;;) {
             if (p.item != null || (q = p.prev) == null) {
                 if (o != null && p.next != p && last.casPrev(prev, p)) {
@@ -637,9 +626,6 @@ public class PortableConcurrentDirectDeque<E>
         whileActive:
         do {
             Node<E> prev = x.prev;
-            // assert prev != null;
-            // assert x != NEXT_TERMINATOR;
-            // assert x != PREV_TERMINATOR;
             Node<E> p = prev;
             findActive:
             for (;;) {
@@ -668,9 +654,6 @@ public class PortableConcurrentDirectDeque<E>
         whileActive:
         do {
             Node<E> next = x.next;
-            // assert next != null;
-            // assert x != NEXT_TERMINATOR;
-            // assert x != PREV_TERMINATOR;
             Node<E> p = next;
             findActive:
             for (;;) {
