@@ -34,13 +34,13 @@ public class URLUtils {
 
     private static final char PATH_SEPARATOR = '/';
 
-    private static final QueryStringParser QUERY_STRING_PARSER = new QueryStringParser() {
+    private static final QueryStringParser QUERY_STRING_PARSER = new QueryStringParser('&', false) {
         @Override
         void handle(HttpServerExchange exchange, String key, String value) {
             exchange.addQueryParam(key, value);
         }
     };
-    private static final QueryStringParser PATH_PARAM_PARSER = new QueryStringParser() {
+    private static final QueryStringParser PATH_PARAM_PARSER = new QueryStringParser(';', true) {
         @Override
         void handle(HttpServerExchange exchange, String key, String value) {
             exchange.addPathParam(key, value);
@@ -66,8 +66,8 @@ public class URLUtils {
         parsePathParams(string, exchange, charset, doDecode, maxParameters);
     }
 
-    public static void parsePathParams(final String string, final HttpServerExchange exchange, final String charset, final boolean doDecode, int maxParameters) throws ParameterLimitException {
-        PATH_PARAM_PARSER.parse(string, exchange, charset, doDecode, maxParameters);
+    public static int parsePathParams(final String string, final HttpServerExchange exchange, final String charset, final boolean doDecode, int maxParameters) throws ParameterLimitException {
+        return PATH_PARAM_PARSER.parse(string, exchange, charset, doDecode, maxParameters);
     }
 
     /**
@@ -204,11 +204,11 @@ public class URLUtils {
                                 bytes[pos++] = (byte) c;
                             } else {
                                 bytes[pos++] = (byte) c;
-                                if (i < numChars) {
-                                    c = s.charAt(i);
-                                }
                             }
 
+                            if (i < numChars) {
+                                c = s.charAt(i);
+                            }
                         }
                     }
 
@@ -238,23 +238,32 @@ public class URLUtils {
 
     private abstract static class QueryStringParser {
 
-        void parse(final String string, final HttpServerExchange exchange, final String charset, final boolean doDecode, int max) throws ParameterLimitException {
+        private final char separator;
+        private final boolean parseUntilSeparator;
+
+        QueryStringParser(final char separator, final boolean parseUntilSeparator) {
+            this.separator = separator;
+            this.parseUntilSeparator = parseUntilSeparator;
+        }
+
+        int parse(final String string, final HttpServerExchange exchange, final String charset, final boolean doDecode, int max) throws ParameterLimitException {
             int count = 0;
+            int i = 0;
             try {
                 int stringStart = 0;
                 String attrName = null;
-                for (int i = 0; i < string.length(); ++i) {
+                for (i = 0; i < string.length(); ++i) {
                     char c = string.charAt(i);
                     if (c == '=' && attrName == null) {
                         attrName = string.substring(stringStart, i);
                         stringStart = i + 1;
-                    } else if (c == '&') {
+                    } else if (c == separator) {
                         if (attrName != null) {
                             handle(exchange, decode(charset, attrName, doDecode), decode(charset, string.substring(stringStart, i), doDecode));
                             if(++count > max) {
                                 throw UndertowMessages.MESSAGES.tooManyParameters(max);
                             }
-                        } else {
+                        } else if (stringStart != i) { // Ignore if attrName == null and stringStart == i because it means both key and value are empty.
                             handle(exchange, decode(charset, string.substring(stringStart, i), doDecode), "");
                             if(++count > max) {
                                 throw UndertowMessages.MESSAGES.tooManyParameters(max);
@@ -262,15 +271,17 @@ public class URLUtils {
                         }
                         stringStart = i + 1;
                         attrName = null;
+                    } else if (parseUntilSeparator && (c == '?' || c == '/')) {
+                        break;
                     }
                 }
                 if (attrName != null) {
-                    handle(exchange, decode(charset, attrName, doDecode), decode(charset, string.substring(stringStart, string.length()), doDecode));
+                    handle(exchange, decode(charset, attrName, doDecode), decode(charset, string.substring(stringStart, i), doDecode));
                     if(++count > max) {
                         throw UndertowMessages.MESSAGES.tooManyParameters(max);
                     }
                 } else if (string.length() != stringStart) {
-                    handle(exchange, decode(charset, string.substring(stringStart, string.length()), doDecode), "");
+                    handle(exchange, decode(charset, string.substring(stringStart, i), doDecode), "");
                     if(++count > max) {
                         throw UndertowMessages.MESSAGES.tooManyParameters(max);
                     }
@@ -278,6 +289,7 @@ public class URLUtils {
             } catch (UnsupportedEncodingException e) {
                 throw new RuntimeException(e);
             }
+            return i;
         }
 
         private String decode(String charset, String attrName, final boolean doDecode) throws UnsupportedEncodingException {
