@@ -81,6 +81,10 @@ public class LoadBalancerConnectionPoolingTestCase {
     @AfterClass
     public static void after() {
         undertow.stop();
+        // sleep 1 s to prevent BindException (Address already in use) when running the CI
+        try {
+            Thread.sleep(1000);
+        } catch (InterruptedException ignore) {}
     }
 
     @Test
