@@ -23,6 +23,7 @@ import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -31,6 +32,7 @@ import java.util.HashSet;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Supplier;
 
 import javax.servlet.ServletException;
 import javax.servlet.ServletOutputStream;
@@ -41,19 +43,24 @@ import javax.servlet.http.HttpSession;
 
 import io.undertow.UndertowLogger;
 import io.undertow.server.HttpServerExchange;
+import io.undertow.server.protocol.http.HttpAttachments;
 import io.undertow.servlet.UndertowServletMessages;
 import io.undertow.servlet.handlers.ServletRequestContext;
 import io.undertow.util.CanonicalPathUtils;
 import io.undertow.util.DateUtils;
+import io.undertow.util.HeaderMap;
 import io.undertow.util.HeaderValues;
 import io.undertow.util.Headers;
 import io.undertow.util.HttpString;
+import io.undertow.util.Protocols;
 import io.undertow.util.RedirectBuilder;
 import io.undertow.util.StatusCodes;
 
+import static io.undertow.util.URLUtils.isAbsoluteUrl;
 
 /**
  * @author Stuart Douglas
+ * @author <a href="mailto:ropalka@redhat.com">Richard Opalka</a>
  */
 public final class HttpServletResponseImpl implements HttpServletResponse {
 
@@ -77,6 +84,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
     private boolean charsetSet = false; //if a content type has been set either implicitly or implicitly
     private String contentType;
     private String charset;
+    private Supplier<Map<String, String>> trailerSupplier;
 
     public HttpServletResponseImpl(final HttpServerExchange exchange, final ServletContextImpl servletContext) {
         this.exchange = exchange;
@@ -89,12 +97,12 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
     }
 
     @Override
-    public void addCookie(final Cookie cookie) {
+    public void addCookie(final Cookie newCookie) {
         if (insideInclude) {
             return;
         }
-        final ServletCookieAdaptor servletCookieAdaptor = new ServletCookieAdaptor(cookie);
-        if (cookie.getVersion() == 0) {
+        final ServletCookieAdaptor servletCookieAdaptor = new ServletCookieAdaptor(newCookie);
+        if (newCookie.getVersion() == 0) {
             servletCookieAdaptor.setVersion(servletContext.getDeployment().getDeploymentInfo().getDefaultCookieVersion());
         }
         exchange.setResponseCookie(servletCookieAdaptor);
@@ -184,13 +192,14 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         resetBuffer();
         setStatus(StatusCodes.FOUND);
         String realPath;
-        if (location.contains("://")) {//absolute url
+        if (isAbsoluteUrl(location)) {//absolute url
             exchange.getResponseHeaders().put(Headers.LOCATION, location);
         } else {
             if (location.startsWith("/")) {
                 realPath = location;
             } else {
-                String current = exchange.getRelativePath();
+                //Match AsyncContext.dispatch, could use 'exchange.getResolvedPath().length()' instead of servlet context
+                String current = exchange.getRequestURI().substring(getServletContext().getContextPath().length());
                 int lastSlash = current.lastIndexOf("/");
                 if (lastSlash != -1) {
                     current = current.substring(0, lastSlash + 1);
@@ -317,10 +326,20 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
 
     @Override
     public String getCharacterEncoding() {
-        if (charset == null) {
-            return servletContext.getDeployment().getDefaultCharset().name();
+        if (charset != null) {
+            return charset;
         }
-        return charset;
+        // first check, web-app context level default response encoding
+        if (servletContext.getDeployment().getDeploymentInfo().getDefaultResponseEncoding() != null) {
+            return servletContext.getDeployment().getDeploymentInfo().getDefaultResponseEncoding();
+        }
+        // now check the container level default encoding
+        if (servletContext.getDeployment().getDeploymentInfo().getDefaultEncoding() != null) {
+            return servletContext.getDeployment().getDeploymentInfo().getDefaultEncoding();
+        }
+        // if no explicit encoding is specified, this method is supposed to return ISO-8859-1 as per the
+        // expectation of this API
+        return StandardCharsets.ISO_8859_1.name();
     }
 
     @Override
@@ -387,11 +406,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
 
     @Override
     public void setContentLength(final int len) {
-        if (insideInclude || responseStarted()) {
-            return;
-        }
-        exchange.getResponseHeaders().put(Headers.CONTENT_LENGTH, Integer.toString(len));
-        this.contentLength = (long) len;
+        setContentLengthLong((long) len);
     }
 
     @Override
@@ -399,7 +414,11 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         if (insideInclude || responseStarted()) {
             return;
         }
-        exchange.getResponseHeaders().put(Headers.CONTENT_LENGTH, Long.toString(len));
+        if(len >= 0) {
+            exchange.getResponseHeaders().put(Headers.CONTENT_LENGTH, Long.toString(len));
+        } else {
+            exchange.getResponseHeaders().remove(Headers.CONTENT_LENGTH);
+        }
         this.contentLength = len;
     }
 
@@ -769,4 +788,33 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
     public boolean isTreatAsCommitted() {
         return treatAsCommitted;
     }
+
+    @Override
+    public void setTrailerFields(Supplier<Map<String, String>> supplier) {
+        if(exchange.isResponseStarted()) {
+            throw UndertowServletMessages.MESSAGES.responseAlreadyCommited();
+        }
+        if(exchange.getProtocol() == Protocols.HTTP_1_0) {
+            throw UndertowServletMessages.MESSAGES.trailersNotSupported("HTTP/1.0 request");
+        } else if(exchange.getProtocol() == Protocols.HTTP_1_1) {
+            if(exchange.getResponseHeaders().contains(Headers.CONTENT_LENGTH)) {
+                throw UndertowServletMessages.MESSAGES.trailersNotSupported("not chunked");
+            }
+        }
+        this.trailerSupplier = supplier;
+        exchange.putAttachment(HttpAttachments.RESPONSE_TRAILER_SUPPLIER, () -> {
+            HeaderMap trailers = new HeaderMap();
+            Map<String, String> map = supplier.get();
+            for(Map.Entry<String, String> e : map.entrySet()) {
+                trailers.put(new HttpString(e.getKey()), e.getValue());
+            }
+            return trailers;
+        });
+    }
+
+    @Override
+    public Supplier<Map<String, String>> getTrailerFields() {
+        return trailerSupplier;
+    }
+
 }
